////////////////////////////////////////////////////////////////////////////////
// PushBlock a java game. Try to move the blue spheres to a read shaded area.
//
// Copyright (C) 1999 Anders Reggestad andersr@pvv.ntnu.no
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////////

import java.awt.*;
import java.util.*;

public class PushBlockBoard {

  public static final int OUTSIDE     = 0; 
  public static final int FLOOR       = 1; 
  public static final int WALL        = 2;
  public static final int GOAL        = 3;
  public static final int BLUE_SPHERE = 4; 
  public static final int PLAYER      = 5; 
  public static final int WATER       = 6; 
  public static final int DIRT        = 7; 
  public static final int FRICTION    = 8; 

  int              m_width;
  int              m_height;
  int              m_board[][];
  int              m_posX = 0;
  int              m_posY = 0;
  int              m_imgWidth  = 0;
  int              m_imgHeight = 0;
  Vector           m_piece; // Vector of elements with type PushBlockPiece
  int              m_moves;
  int              m_level;
  int              m_nextLevel;
  String           m_info;

  public PushBlockBoard(int width, int height, int level, int nextLevel){
    m_width = width;
    m_height = height;
    m_board = new int [m_width][m_height];
    m_level = level;
    m_nextLevel = nextLevel;
    m_piece = new Vector(0,1); // initialCapacity = 0, capacityIncrement = 1
    for (int ii = 0; ii < m_width; ii++){
      for (int jj = 0; jj < m_height; jj++){
        m_board[ii][jj] = FLOOR;
      }
    }
    m_moves = 0;
  }

  public void setImageSize(int imgWidth,int imgHeight){
    m_imgWidth = imgWidth;
    m_imgHeight = imgHeight;
  }

  public void paint(Graphics g,Image[] images){
    for (int ii = 0; ii < 9; ii++){
      for (int jj = 0; jj < 9; jj++){
        if ( ((ii+m_posX-4) >= 0) && ((ii+m_posX-4) < m_width) &&
             ((jj+m_posY-4) >= 0) && ((jj+m_posY-4) < m_height) ){
          g.drawImage(images[m_board[ii+m_posX-4][jj+m_posY-4]],
                      ii*m_imgWidth,m_imgHeight*jj,null);
        } else {
          g.drawImage(images[OUTSIDE],ii*m_imgWidth,
                      m_imgHeight*jj,null);
        }
      }
    }
    g.drawImage(images[PLAYER],4*m_imgWidth,4*m_imgHeight,null);
    for (int ii = 0; ii < m_piece.size(); ii++){
      Image pieceImage;
      PushBlockPiece piece = (PushBlockPiece)m_piece.elementAt(ii);
      switch (piece.getType()){
      case PushBlockPiece.DIRT: pieceImage = images[DIRT]; break;
      default: pieceImage = images[BLUE_SPHERE]; break;
      }
      g.drawImage(pieceImage,
                  (piece.getPosX()-m_posX+4)*m_imgWidth,
                  (piece.getPosY()-m_posY+4)*m_imgHeight,null);
    }
  }

  public void setTile(int x,int y,int tile){
    m_board[x][y] = tile;
  }

  public void addPiece(int x,int y,int type){
    m_piece.addElement(new PushBlockPiece(x,y,type));
  }

  public void setPosition(int posX,int posY){
    m_posX = posX;
    m_posY = posY;
  }

  boolean isLegal(int x,int y){
    int board = m_board[x][y];
    return ((board == FLOOR)||(board==GOAL)||(board==FRICTION));
  }

  boolean isGoal(int x,int y){
    return (m_board[x][y] == GOAL);
  }

  boolean hasFriction(int x,int y){
    return (m_board[x][y] == FRICTION);
  }

  int isFree(int x,int y){
    for (int ii = 0; ii < m_piece.size(); ii++){
      PushBlockPiece piece = (PushBlockPiece)m_piece.elementAt(ii);
      if ((piece.getPosX() == x) && (piece.getPosY() == y)){
        return ii;
      }
    }
    return -1;
  }

  boolean isInside(int xx,int yy){
    return ( (xx>=0) && (yy >= 0) && (xx < m_width) && (yy < m_height) );
  }
  
  public void move(int dx,int dy){
    int xx = m_posX+dx;
    int yy = m_posY+dy;

    if ( !isInside(xx,yy) || !isLegal(xx,yy)) return;

    int index = isFree(xx,yy);
    if (index != -1){
      if (!isInside(xx+dx,yy+dy) || !isLegal(xx+dx,yy+dy) || 
          !(isFree(xx+dx,yy+dy)==-1)) return;

      PushBlockPiece piece = (PushBlockPiece)m_piece.elementAt(index);

      if ((piece.getType() == PushBlockPiece.DIRT) &&
          hasFriction(xx+dx,yy+dy)) return;

      piece.move(dx,dy);
    }       

    m_posX+=dx;
    m_posY+=dy; 
    m_moves++;
  }

  public boolean isFinished(){
    for (int ii = 0; ii < m_piece.size(); ii++){
      PushBlockPiece piece = (PushBlockPiece)m_piece.elementAt(ii);
      if ( (piece.getType() == PushBlockPiece.SPHERE) && 
           !isGoal(piece.getPosX(),piece.getPosY()))
        return false;
    }
    return true;
  }

  public void moveUp(){
    move(0,-1);
  }

  public void moveDown(){
    move(0,1);
  }

  public void moveLeft(){
    move(-1,0);
  }

  public void moveRight(){
    move(1,0);
  }

  public int getScore(){
    return m_moves;
  }

  public int getNextLevel(){
    return m_nextLevel;
  }

  public void setInfo(String info){
    m_info = info;
  }

  public String getInfo(){
    return m_info;
  }

}
